function out = f(u)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab M-file                Author: Christophe Prieur
% from a file of Ricardo Sanfelice
%
% Project: Simulation of a hybrid system
%
%
% Name: f.m
%
% Description: Flow map
%
% Version: 0.3
% Required files: -
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% constants
global eps0a eps1a ep0b eps1b option_simu control_values;

% state
x1 = u(1);
x2 = u(2);
zeta1 = u(3);
z0 = u(4);
z1 = u(5);
q= u(6);

% computation of the static controller

% output of the system
y = x1;

if option_simu == 1
    %%% when the nonlinear system is simulated
    control = zeta1;
end

if option_simu == 2
    %%% when the hybrid system is simulated
    if q == 1
        % when the global controller should be used
        control = zeta1;
    end
    if q == 0
        % when the local controller should be used
        control = 0;
    end
end

x1dot = -x1+ (control -x2)* x1^2;
x2dot = -x2+ x1^2;

    
if option_simu == 1
    %%% when the nonlinear system is simulated  
    zeta1dot = -zeta1+ y^2 - 2 *y^5;
    z0dot = 0;  % other variables remain constant
    z1dot = 0;  
    qdot = 0;
end

if option_simu == 2
    %%% when the hybrid system is simulated  
    zeta1dot = q*(-zeta1+ y^2 - 2 *y^5);
    z0dot = (1-q)*(-z0+3/2*y^6+y^4);  
    z1dot = -z1+0.5*control^2+ y^4+3/2*y^6;  
    qdot = 0;
end

control_values=[control_values;control]; 
        
out = [x1dot;x2dot;zeta1dot;z0dot;z1dot;qdot];