function plotC(xmin,xmax,res,color,linetype)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab M-file                Author: Christophe Prieur
%    from a m file created by Ricardo Sanfelice
%
% Project: Simulation of a hybrid system
%
%
% Name: plotC.m
%
% Description: plots the flow set
%
% variables
% in 1D-case (D=dimension)
%       xmin: left corner, 
%       xmax: right corner, res: resolution
% in 2D-case
%       xmin: left down corner, 
%       xmax: right top corner, 
% in 3D-case
%       xmin: left down corner of the down face, 
%       xmax: right top corner of the up face, 
%
%       res: resolution (thus res^D points are plotted)
%
%       color (optional, default = 'b')
%       
%       linetype (optional, default = '*')
%
% 
% Version: 0.3
% Required files: C.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% get dimensions
d = length(xmin);
%figure(3)
%clf

if nargin==3
 linetype='*'; color = 'b';
else
 color='b';  
end

if (d==1)
    xin = linspace(xmin,xmax,res);
    N = length(xin);
    for i=1:N,   
            %     Cout(i) = C(xin(i));
            if (C([xin(i),0,0]) == 1)
                plot(xin(i),'b*')
                hold on
            end
   end
elseif (d==2)
    % keyboard
    
    % compute the corners
    corner.ld = xmin; % left down corner
    corner.rd = [xmax(1),xmin(2)]; % right down corner
    corner.rt = xmax; %right top corner
    corner.lt = [xmin(1),xmax(2)]; % left top corner
    
    % compute the boundaries
    boundary.d = [linspace(corner.ld(1),corner.rd(1),res)]; % x donwn boundary 
    %boundary.t = [linspace(corner.lt(1),corner.rt(1),res)]; % x top boundary
    %boundary.r = [linspace(corner.rd(2),corner.rt(2),res)]; % y right boundary 
    boundary.l = [linspace(corner.ld(2),corner.lt(2),res)]; % y left boundary
    
    for i=1:res
        for ii=1:res,   
            %point = [boundary.d(i);boundary.l(ii)];  % current point
            if (C([boundary.d(i),boundary.l(ii),0]) == 1)
                plot(boundary.d(i),boundary.l(ii),[color linetype])
                hold on
            end
        end
    end
    axis([xmin(1), xmax(1), xmin(2), xmax(2)])
    xlabel('x1');
    ylabel('x2');
    
elseif (d==3)
%     % down-face
%     down.ld = xmin; % down-face and left down  corner
%     down.rd = [xmax(1),xmin(2),xmin(3)]; % right down corner
%     down.rt = [xmax(1),xmax(2),xmin(3)]; %right top corner
%     down.lt = [xmin(1),xmax(2),xmin(3)]; % left top corner
%     % up-face
%     up.ld = [xmin(1),xmin(2),xmax(3)];; % down-face and left  corner
%     up.rd = [xmax(1),xmin(2),xmax(3)]; % right down corner
%     up.rt = xmax; %right top corner
%     up.lt = [xmin(1),xmax(2),xmax(3)]; % left top corner
    
    % compute three lines
    boundary.d = linspace(xmin(1),xmax(1),res); % x1 direction
    boundary.t = linspace(xmin(2),xmax(2),res); % x2 direction
    boundary.c = linspace(xmin(3),xmax(3),res); % x3 direction
    %keyboard
    
    for i=1:res
        for ii=1:res,   
            for iii=1:res,
                % point=
                %keyboard
                if (C([boundary.d(i),boundary.t(ii),boundary.c(iii)]) == 1)
                    plot3(boundary.d(i),boundary.t(ii),boundary.c(iii),[color linetype]);
                    hold on
                end
            end
        end
    end
    xlabel('x1');
    ylabel('x2');
    zlabel('x3');
end
%title('plot of the set C');


