function plotD(xmin,xmax,res,color,linetype)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab M-file                Author: Christophe Prieur
%    from a m file created by Ricardo Sanfelice
%
% Project: Simulation of a hybrid system
%
%
% Name: plotD.m
%
% Description: plots the jump set
%
% variables
% in 1D-case (D=dimension)
%       xmin: left corner, 
%       xmax: right corner, res: resolution
% in 2D-case
%       xmin: left down corner, 
%       xmax: right top corner, 
%
%       res: resolution (thus res^D points)
%
%       color (optional, default = 'r')
%       
%       linetype (optional, default = '*')
%
% 
% Version: 0.3
% Required files: D.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% get dimensions
d = length(xmin);
%figure(3)
%clf


if nargin==3
 linetype='*'; color = 'b';
else
 color='b';  
end

if (d==1)
   xin = linspace(xmin,xmax,res);
   N = length(xin);
   for i=1:N,   
%     Cout(i) = C(xin(i));
     if (D(xin(i)) == 1)
          plot(xin(i),'r*')
          hold on
     end
   end
elseif (d==2)
    % keyboard
    
    % compute the corners
    corner.ld = xmin; % left down corner
    corner.rd = [xmax(1),xmin(2)]; % right down corner
    corner.rt = xmax; %right top corner
    corner.lt = [xmin(1),xmax(2)]; % left top corner
    
    % compute the boundaries
    boundary.d = [linspace(corner.ld(1),corner.rd(1),res)]; % x donwn boundary 
    %boundary.t = [linspace(corner.lt(1),corner.rt(1),res)]; % x top boundary
    %boundary.r = [linspace(corner.rd(2),corner.rt(2),res)]; % y right boundary 
    boundary.l = [linspace(corner.ld(2),corner.lt(2),res)]; % y left boundary
    
    for i=1:res
        for ii=1:res,   
            %point = [boundary.d(i);boundary.l(ii)];  % current point
            if (D([boundary.d(i),boundary.l(ii),0]) == 1)
                plot(boundary.d(i),boundary.l(ii),'r*')
                hold on
            end
        end
    end
    axis([xmin(1), xmax(1), xmin(2), xmax(2)]);
    xlabel('x1');
    ylabel('x2');
end
title('plot of the set D');
