% in this m-file the hybrid system for the non-linear example is simulated
% and the plots are drawn
%
% author: C Prieur
% for the [CP, A. Teel, IEEE-TAC, submitted] paper
%%%%%%%%%%%%%%%%%%%%


% clear all


% initial conditions and time horizon
%%%%%%%%%%%%%%%%%%%%
option_initial = 1; % interesting solution starting by using the local
%option_initial = 2; % interesting solution starting by using the global

if option_initial == 1  
% for the plant
xp0 = [1;-1];
% for the dynamical state of the global controller
zeta10 = 0;
% for the local norm-observer
z00 = 0;
% for the global norm-observer
z10 = 0;
% for the discrete value
q0 = 0; % should be either 0 or 1
% simulation horizon
T = 4;
%T = 0.1;
J  = 500;
end


if option_initial == 2  
% for the plant
xp0 = [0.3;-0.3];
% for the dynamical state of the global controller
zeta10 = 1;
% for the local norm-observer
z00 = 0;
% for the global norm-observer
z10 = 1;
% for the discrete value
q0 = 1; % should be either 0 or 1
% simulation horizon
T = 3.5;
%T = 0.1;
J  = 500;
end


% simple rules to be sure that the initial conditions are compatible
if (q0== 1) & (z00~=0)
    z00 = 0;
    disp('modification of the value of z00')
end

if (q0== 0) & (zeta10~=0)
    zeta10 = 0;
    disp('modification of the value of zeta10')
end

% stack all initial states
x0=[xp0;zeta10;z00;z10;q0];

% constants
% global mu;


% rule for jumps
% rule = 1 -> priority for jumps
% rule = 2 -> priority for flows
% rule = 3 -> no priority, random selection when simultaneous conditions
rule = 1;

% simulate
sim('HybridSimulator')

% plots
x1 = x(:,1);
x2 = x(:,2);
zeta1 = x(:,3);
z0 = x(:,4);
z1 = x(:,5);
q= x(:,6);

n_fig = 0; 

% plots for x1
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,x1)
grid on
ylabel('x1')

subplot(2,1,2),plotjumps(t,j,x1)
grid on
ylabel('x1')

% plots for x2
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,x2)
grid on
ylabel('x2')

subplot(2,1,2),plotjumps(t,j,x2)
grid on
ylabel('x2')

% plots for zeta1
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,zeta1)
grid on
ylabel('zeta1')

subplot(2,1,2),plotjumps(t,j,zeta1)
grid on
ylabel('zeta1')

% plots for z0
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,z0)
grid on
ylabel('z0')

subplot(2,1,2),plotjumps(t,j,z0)
grid on
ylabel('z0')

% plots for z1
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,z1)
grid on
ylabel('z1')

subplot(2,1,2),plotjumps(t,j,z1)
grid on
ylabel('z1')


% plots for q
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,q)
grid on
ylabel('q')

subplot(2,1,2),plotjumps(t,j,q)
grid on
ylabel('q')

% plots of the control_values
n_fig=n_fig+1;
figure(n_fig)
clf
tt=linspace(0,T,size(control_values,1));
plot(tt,control_values,'LineWidth',1.5)
grid on
ylabel('u')
xlabel('t')
print -depsc2 u_nonlinear.eps

% plots of the x1 x2 in the same figure
n_fig=n_fig+1;
figure(n_fig)
clf
hold on 
plotflows(t,j,x1,'b','-')
plotflows(t,j,x2,'r','-.')
xlabel('t')
ylabel('x')
axis tight
grid on
print -depsc2 x_nonlinear.eps

% plots for x2
n_fig=n_fig+1;
figure(n_fig)
clf
subplot(2,1,1),plotflows(t,j,x2)
grid on
ylabel('x2')


% plots of the (x1,x2) plane
n_fig=n_fig+1;
figure(n_fig)
clf
plot(x1,x2)
grid on
xlabel('x1')
ylabel('x2')

break


%%%%%%%%%%%%
%%%%%%%%%%%

n_fig=n_fig+1;
figure(n_fig)
clf
subplot(3,1,1),plotflows(t,j,x1)
%axis([0 1 0 10]);
axis tight
grid on
ylabel('x1')

subplot(3,1,2),plotflows(t,j,x2)
axis tight
grid on
ylabel('x2')

subplot(3,1,3),plotflows(t,j,x3)
axis tight
grid on
ylabel('x3')

figure(5)
clf
subplot(3,1,1),plotflows(t,j,x1)
%axis([0 1 0 10]);
axis tight
grid on
ylabel('x1')

subplot(3,1,2),plotflows(t,j,x2)
axis tight
grid on
ylabel('x2')

% backstepping controller
phi= -mu*(x2+nu*x1+x1.^2)-x1-x1.^2-nu*(x2+x1.^2)-2*x1.*(x2+x1.^2);

subplot(3,1,3),plotflows(t,j,phi)
axis tight
grid on
ylabel('phi')
print -depsc2  nonlinear


%break
% plot sets
% plot the intersection with the plane x3=3
figure(6),clf,plotC([-10,-10],[10,10],100);%, hold on, plotD(-1,10,100)
print -depsc2  shape-flow-nonlinear
figure(7),clf,plotD([-10,-10],[10,10],100);%, hold on, plotD(-1,10,100)
figure(8)
plot(x(:,1),x(:,2),'r')

N_part=400; % when the x3 is small 
disp(['at time t=', num2str(t(N_part))] )
disp(['value of x3=',num2str(x(N_part,3))])
%x(N_part,3)

figure(9),clf,plotC([-10,-700],[30,100],300,'b','.');
hold on
plot(x(N_part:size(t,1),1),x(N_part:size(t,1),2),'rx')
print -depsc2  shape-flow-solution-nonlinear

% plot the flow set in 3D
N_part=3000; % when the x3 is small 
disp(['at time t=', num2str(t(N_part))] )
disp(['value of x3=',num2str(x(N_part,3))])
%figure(10), clf, plotC([-2,-10,-10],[10,10,10],30); view(-10,40);
figure(10), clf, plotC([-1,-10,-10],[10,10,10],45); view(-16,30);
hold on
plot3(x(N_part:size(t,1),1),x(N_part:size(t,1),2),x(N_part:size(t,1),3),'rx')
print -depsc2  shape-flow-solution-nonlinear-3d

break

res=20;
x = linspace(-10,10,res);
y = linspace(-10,10,res);

figure(11), clf
for i =1:res
    for ii=1:res
        %keyboard
        plot3(x(i),y(ii),bdryC(x(i),y(ii)),'b*')
        hold on
    end
end

break
% plot hybrid arc
plotHybridArc(t,j,x)
xlabel('j')
ylabel('t')
zlabel('x')

