% second example of the paper 
% C Prieur, A. Teel,
% Uniting local and global output feedback controllers

clear all
pause off
close all


% see also the simulink file : evolution.mdl

% load the control system
% \dot x = A x + B sat_1 (u)
% y = C x

% saturation level
u1=10;
example ='first';

switch example
    case 'first'
% plant
a = 1;
b =0.1;
plant = tf(a, [1 b 0]);

% controller
%k=-30;
k=-10;
z1=1; z2=1; p=14;
local= tf([k k*(z1+z2) k*z1*z2],[1 p 0]);

closed_l= feedback(plant,local,+1);

if not(isstable(closed_l))
    disp('instable')
end


[A_cl,B_cl,C_cl,D_cl] = ssdata(closed_l);
%rlocus(closed_l);

disp('eigenvalues and eigenvectors of the local closed loop syst');
damp(closed_l)

[A_p,B_p,C_p,D_p] = ssdata(plant);
% put the plant in the same coordinates than the paper
% change the order the lines:
A_p= [0 1; 1 0]*A_p;
B_p= [0 1; 1 0]*B_p;
% change the order of the columns:
A_p= A_p*[0 1; 1 0];
C_p= C_p*[0 1; 1 0];

[A_l,B_l,C_l,D_l] = ssdata(local);

    case 'second'
    disp('to be continued')

end

% initial condition for the plant
x0_p=10*[1 ;1];

% initial condition for the local controller
x0_l=10*[1 ;1];

% global gain
%K=-50; %or K=-100 with u1=1.5
K=-10^(-1);
closed_g= feedback(plant,K,+1);

% simulations with the initial condition
% use the simulink to simulate the system
% and
% plot(t,y)
% Stop time for the simulink model
Tf=100;
% error for the global controller
error=0;
disp('use evolution.mdl to simulate the system')
%
% end of simulations

disp('the eigenvalues of the global controller in linear form are:');
disp(eig(A_p+ B_p*(K*C_p)));

pause


%%
% now we focus on the uniting algorithm

eps3= 1;


n.n_p = size(A_p,2); % number of states
n.m_p = size(B_p,2); % number of commands
n.y_p = size(C_p,1); % number of output

P1 = sdpvar(n.n_p,n.n_p,'symmetric');
M =  sdpvar(n.n_p,n.y_p,'full');        % M is P1*L
%eps3= sdpvar(1,1,'full');

MI1 = P1*A_p + M*C_p+ A_p'*P1 + C_p'*M' + 2*P1;
MI2 = P1;
%MI3 = eps3;

LMI = set(MI1<=0)+ set(MI2>0);% +set(MI3>0);

solvesdp(LMI,[],sdpsettings('solver','sedumi'));
checkset(LMI);

%eps3=double(eps3);
P1= double(P1); L=inv(P1)*double(M);

if min(eig(P1))<0
    disp('problem computation P1')
end

%W1 = inv(P1);

disp('end of computation of P1');
pause


%%
% computation of a P0 for the local system

n.n_l = size(A_l,2); % number of states
n.m_l = size(B_l,2); % number of commands
n.y_l = size(C_l,1); % number of output

n.n_cl = size(A_cl,2); % number of states
n.m_cl = size(B_cl,2); % number of commands
n.y_cl = size(C_cl,1); % number of output


W0 = sdpvar(n.n_cl,n.n_cl,'symmetric');
if n.m_cl ~= 1
    disp('particularize to this case'); return;
end

Z =  sdpvar(1,n.m_cl,'full');
S = sdpvar(n.m_cl,1,'full');
mu = sdpvar(1,1,'full');

AA = [[A_p    , zeros(n.n_p,n.n_l) ];...
      [B_l*C_p, A_l]];
BB= [B_p; zeros(n.n_l,n.m_l)];
KK= [D_l*C_p, C_l];
AAA= AA+BB*KK;

MI1=blkvar;
MI1(1,1)=W0*AAA'+AAA*W0 ;
MI1(1,2)=BB*diag(S)- Z';
MI1(2,2)=-2*diag(S);
MI1=sdpvar(MI1);

MI2=blkvar;
MI2(1,1)=W0;
MI2(1,2)=W0*KK'-Z';
MI2(2,2)=u1^2;
MI2=sdpvar(MI2);

% for the objective (maximize the size of the estimation of the basin of attraction)

MI3=blkvar; % minimize mu to make P0 small
MI3(1,1)=mu*eye(n.n_cl);
MI3(1,2)=eye(n.n_cl);
MI3(2,2)=W0;
MI3=sdpvar(MI3);

% solve
LMI = set(MI1<0)+set(MI2>=0)+set(MI3>0);

solvesdp(LMI,mu,sdpsettings('solver','lmilab'));
checkset(LMI);

W0=double(W0);S=double(S); Z=double(Z); mu=double(mu);

test1=[ W0*AAA'+AAA*W0  BB*diag(S)- Z';
    (BB*diag(S)- Z')' -2*diag(S)];
if max(eig(test1))>=0 
    disp('problem test1')
end
test2=[W0 W0*KK'-Z';
    (W0*KK'-Z')' u1^2];
if min(eig(test2))<0
    disp('problem test2')
end
if mu<=0 
    disp('problem test3')
end
P0 = inv(W0);

%eps4=1;

disp('end of the computation of P0 for the local controller')

%%
pause

%%
% computation of R and eps0b

eps = sdpvar(1,1,'full'); % inverse of eps0b
R = sdpvar(n.n_l,n.n_l,'symmetric');

bal =10^0;
MI1 = [eps*P1 , zeros(n.n_p,n.n_l);
       zeros(n.n_l,n.n_p) , R      ]  - bal*P0;
% the 'bal' is to balance the P1 and the P0
% if necessary
MI1=sdpvar(MI1);

% solve
LMI = set(MI1>=0);

solvesdp(LMI,eps+trace(R),sdpsettings('solver','lmilab'));
%solvesdp(LMI,eps,sdpsettings('solver','lmilab'));

checkset(LMI);

eps =  inv(bal)*double(eps);

R = inv(bal)*double(R);

test1=[eps*P1 , zeros(n.n_p,n.n_l);
       zeros(n.n_l,n.n_p) , R      ]  - bal*P0;
if min(eig(test1))<0
    disp('probleme test1 esp0b')
end

eps0b = inv(eps);
Q0 = eps0b*R;

% gap between the epsia and epsib
gap=0.99;
eps0a = gap*eps0b;

disp('end of the computation of Q0, eps0a, and eps0b')
pause

%%  
% computation of H and eps hat
H = sdpvar(n.m_p,n.n_p+n.n_l,'full');
epshat = sdpvar(1,1,'full'); % related to the inverse of eps1b

MI3=blkvar;
MI3(1,1)=epshat*P0;
MI3(1,2)=u1*H';
MI3(2,2)=1;
MI3=sdpvar(MI3);


% solve
LMI =  set(MI3>=0);

solvesdp(LMI,epshat,sdpsettings('solver','lmilab'));
%solvesdp(LMI,epshat,sdpsettings('solver','lmilab'));

checkset(LMI);

epshat = double(epshat);
H = double(H);

test2=[ epshat*P0  u1*H';
       (u1*H')' 1];

if min(eig(test2))<0
    disp('problem test2 computation of H')
end

disp('end of computation of H and epshat')
pause

%%
% computation of  N and P 
% (eps1a, eps1b will be computed at the next optimization)

epstilda = sdpvar(1,1,'full'); % related to the inverse of eps1b

%N = NN;
N= sdpvar(n.m_p+n.y_cl+n.n_l,n.m_p+n.n_l+n.y_cl,'symmetric');
P = sdpvar(n.m_p,n.m_p,'symmetric');

bal=10;

II=blkdiag(eye(n.y_cl),C_p, eye(n.n_l));

MI0=P;
MI1 = N;

MI2=blkvar;
if n.m_p~=1
    disp('modify this part')
end

MI2(1,1)=II(:,1:n.m_p)'*N*II(:,1:n.m_p)-2*bal*P;
MI2(1,2)=II(:,1:n.m_p)'*N*II(:,n.m_p+1:5)-bal*P*(KK-H) ;
MI2(2,2)=II(:,n.m_p+1:5)'*N*II(:,n.m_p+1:5)-bal*epstilda*P0;
MI2=sdpvar(MI2);

M3_11= 2*B_p'*P1*B_p;
M3_12 = 2*B_p'*P1*B_p*D_l;
M3_13 = 2*B_p'*P1*B_p*C_l;
M3_22 = 2*(L'*P1*L+D_l*B_p'*P1*B_p*D_l);
M3_23= D_l'*B_p'*P1*B_p*C_l+B_l'*Q0;
M3_33 = 2*C_l'*B_p'*P1*B_p*C_l+A_l'*Q0+Q0*A_l+Q0;
M3=[ M3_11  M3_12  M3_13;
     M3_12' M3_22  M3_23;
     M3_13' M3_23' M3_33];
MI3=bal*M3-N;
MI3=sdpvar(MI3);

% solve
LMI = set(MI2<0)+set(MI3<=0)+set(MI1>=0);%+set(MI0>0)

%solvesdp(LMI,trace(diag(N)),sdpsettings('solver','sedumi'));
solvesdp(LMI,epstilda,sdpsettings('solver','lmilab'));
checkset(LMI);

epstilda=double(epstilda);
NN=double(N);N=double(N)/bal;
%N=double(N)/bal;
P=double(P);

% now we check the lmis of P and N

if min(eig(P))<0 
    disp('problem computation of P') 
end
if min(eig(N))<0 
    disp('problem computation of N') 
end

test2_11=II(:,1:n.m_p)'*NN*II(:,1:n.m_p)-bal*2*P;
test2_12=II(:,1:n.m_p)'*NN*II(:,n.m_p+1:5)-bal*P*(KK-H);
test2_22=II(:,n.m_p+1:5)'*NN*II(:,n.m_p+1:5)-bal*epstilda*P0;

test2 = [test2_11    test2_12;
         (test2_12)' test2_22];
if max(eig(test2))>=0
    disp('problem test2 for the computation of P')
end
 
test3_11= 2*B_p'*P1*B_p;
test3_12 = 2*B_p'*P1*B_p*D_l;
test3_13 = 2*B_p'*P1*B_p*C_l;
test3_22 = 2*(L'*P1*L+D_l*B_p'*P1*B_p*D_l);
test3_23= D_l'*B_p'*P1*B_p*C_l+B_l'*Q0;
test3_33 = 2*C_l'*B_p'*P1*B_p*C_l+A_l'*Q0+Q0*A_l+Q0;
test3=[ test3_11  test3_12  test3_13;
        test3_12' test3_22  test3_23;
        test3_13' test3_23' test3_33];
test3=bal*test3 - NN;
if max(eig(test3))>= 0
    disp('problem test3 for the computation of P')
end

disp('end of computation of N, and P')
pause

%%

% computation of eps1a, eps1b

epstilda = sdpvar(1,1,'full'); % try to find a besser eps1a

bal=10000;

II=blkdiag(eye(n.y_cl),C_p, eye(n.n_l));

MI2=blkvar;
if n.m_p~=1
    disp('modify this part')
end

MI2(1,1)=II(:,1:n.m_p)'*N*II(:,1:n.m_p)-2*bal*P;
MI2(1,2)=II(:,1:n.m_p)'*N*II(:,n.m_p+1:5)-bal*P*(KK-H) ;
MI2(2,2)=II(:,n.m_p+1:5)'*N*II(:,n.m_p+1:5)-bal*epstilda*P0;
MI2=sdpvar(MI2);

% solve
LMI = set(MI2<=0);

%solvesdp(LMI,trace(diag(N)),sdpsettings('solver','sedumi'));
solvesdp(LMI,epstilda*10000,sdpsettings('solver','lmilab'));
checkset(LMI);

epstilda=double(epstilda);

eps1b=min([eps0b*eps3,eps0b*eps0a/epstilda,eps0b/epshat]);
eps1a = gap*eps1b;

% now we check the lmis of P and N

test2_11=II(:,1:n.m_p)'*N*II(:,1:n.m_p)-bal*2*P;
test2_12=II(:,1:n.m_p)'*N*II(:,n.m_p+1:5)-bal*P*(KK-H);
test2_22=II(:,n.m_p+1:5)'*N*II(:,n.m_p+1:5)-bal*epstilda*P0;

test2 = [test2_11    test2_12;
         (test2_12)' test2_22];
if max(eig(test2))>=0
    disp('problem test2 for the computation of P')
end

disp('end of computation of eps1b and eps1a')
pause

%%
% computation of eps2

disp('we donot need to compute eps2')
disp('end of computation of eps2')
pause

%%
% simulation of the closed-loop system
close all;

%%

fig=0;

% initial conditions

inicond = 'b4';
% a1= first figures
% a2 = second figure
% a3 = same as a2 but with more points
% b1, b2, b3, b4 = for the set from which only the local controller is used

% final time for the b1,b2, b3, b4's simulations  
Tb=30;

switch inicond

    case 'a1'
%
x0 = [0;0.05;0;0];
q0=1;z10=0.05;z00=0.05;
%q0=0;z10=0;z00=10^(-10);

% final time
T=100;
% number of discretization
Nt=10000;

  case 'a2'
%
x0 = [0;0.05;0;0];
q0=0;z10=0.05;z00=0.05;
%q0=0;z10=0;z00=10^(-10);

% final time
T=60;
% number of discretization
Nt=10000;

  case 'a3'
%
x0 = [0;0.05;0;0];
q0=0;z10=0.05;z00=0.05;
%q0=0;z10=0;z00=10^(-10);

% final time
T=60;
% number of discretization
Nt=100000;

  case 'b1'
%
x0 = [0.01;-0.01; 0; 0];
q0=0;z10=0.01;z00=0.01;
%q0=0;z10=0;z00=10^(-10);

% final time
T=Tb;
% number of discretization
Nt=1000;

  case 'b2'
%
x0 = [0.01;0.01; 0; 0];
q0=0;z10=0.01;z00=0.01;
%q0=0;z10=0;z00=10^(-10);

% final time
T=Tb;
% number of discretization
Nt=1000;

  case 'b3'
%
x0 = [-0.01;0.01; 0; 0];
q0=0;z10=0.01;z00=0.01;
%q0=0;z10=0;z00=10^(-10);

% final time
T=Tb;
% number of discretization
Nt=1000;

  case 'b4'
%
x0 = [-0.01;-0.01; 0; 0];
q0=0;z10=0.01;z00=0.01;
%q0=0;z10=0;z00=10^(-10);

% final time
T=Tb;
% number of discretization
Nt=1000;

end


% time step
dt=T/Nt;
t=linspace(0,T,Nt);

% simulations of the system in closed-loop with the local controller
if 0
% initial step
x=zeros(4,Nt);y =zeros(1,Nt);u=zeros(1,Nt);
i=1;x(:,i)= x0;y(:,i)=C_p*x0(1:2,i);

while i<Nt
    % we use the local controller
    x(1:2,i+1) = x(1:2,i) + dt* (A_p * x(1:2,i)+ ...
                            B_p* sat(C_l *x(3:4,i) +D_l*C_p* x(1:2,i),u1));
    x(3:4,i+1) = x(3:4,i) + dt* (A_l * x(3:4,i)+  B_l*C_p*x(1:2,i));
    y(:,i+1)=C_p*x(1:2,i);
    i=i+1;%disp(i);
end


% plots
%fig=fig+1;figure(fig);plot(t,x(1,:));ylabel('x1');xlabel('t');title('closed-loop with the local controller')
%fig=fig+1;figure(fig);plot(t,x(2,:));ylabel('x2');xlabel('t');title('closed-loop with the local controller')
end

% simulations of the system in closed-loop with the global controller
if 0
% initial step
x=zeros(4,Nt);y =zeros(1,Nt);u=zeros(1,Nt);
i=1;x(:,i)= x0;y(:,i)=C_p*x0(1:2,i);
z1(i)=z10;

%keyboard

while i<Nt
    % we use the global controller
    uu=sat(K*C_p*x(1:2,i),u1);
    u(:,i)=uu;
    x(1:2,i+1) = x(1:2,i) + dt* (A_p * x(1:2,i)+ B_p*uu);
    x(3:4,i+1) = zeros(2,1);
    y(:,i+1)=C_p*x(1:2,i);
    rho1= u*B_p'*P1*B_p*u+ x(1:2,i)'*C_p'*L'*P1*L*C_p*x(1:2,i);
    rho1=2*rho1;
    z1(i+1)= z1(i) + dt*( -z1(i) + rho1);
    i=i+1;%disp(i);
end

% plots
%fig=fig+1;figure(fig);plot(t,x(1,:));ylabel('x1');xlabel('t');title('closed-loop with the global controller')
%fig=fig+1;figure(fig);plot(t,x(2,:));ylabel('x2');xlabel('t');title('closed-loop with the global controller')
%fig=fig+1;figure(fig);plot(t,z1(:));ylabel('z1');xlabel('t');title('closed-loop with the global controller')
end

% simulations of the system in closed-loop with the hybrid controller

% initial step
x=zeros(4,Nt);z1=zeros(1,Nt);z0=zeros(1,Nt);q=2*ones(1,Nt);
y =zeros(1,Nt);u=zeros(1,Nt);

if q0 == 0
    disp('we start with the local controller')
else
    disp('we start with the global controller')
end

i=1;x(:,i)= x0;z1(i)=z10;z0(i)=z00;q(i)=q0;

evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);

% evo = 1000 => we are in C0
% evo = 100 => we are in C1
% evo = 10 => we are in D0
% evo = 1 => we are in D1

while i<Nt
    while and(evo >= 1000,i<Nt)
        % we use the local controller
        % keyboard
        %disp('local');
    uu= sat(C_l *x(3:4,i) +D_l*C_p* x(1:2,i),u1);
    u(:,i)=uu;
    x(1:2,i+1) = x(1:2,i) + dt* (A_p * x(1:2,i)+ B_p* uu);
    x(3:4,i+1) = x(3:4,i) + dt* (A_l * x(3:4,i)+  B_l*C_p*x(1:2,i));
    q(i+1) = 0;
    rho0=[uu-C_l*x(3:4,i)-D_l*C_p*x(1:2,i),C_p*x(1:2,i),x(3:4,i)']...
        *N*[ uu-C_l*x(3:4,i)-D_l*C_p*x(1:2,i),C_p*x(1:2,i),x(3:4,i)']';
    rho1= uu*B_p'*P1*B_p*uu+ x(1:2,i)'*C_p'*L'*P1*L*C_p*x(1:2,i);
    rho1=2*rho1;
    z0(i+1) = z0(i) + dt*( -z0(i)+ rho0);
    z1(i+1)= z1(i) + dt*( -z1(i) + rho1);
    i=i+1;%disp(i);
    evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);
    end
    while and(and(evo<1000,evo>=100),i<Nt)
        % we use the global controller
        %disp('global');
        uu=sat(K*C_p*x(1:2,i),u1);
        u(:,i)=uu;
        x(1:2,i+1) = x(1:2,i) + dt* (A_p * x(1:2,i)+ B_p*uu);
        x(3:4,i+1) = zeros(2,1);
        q(i+1) = 1;
        z0(i+1) = 0;
        rho1= uu*B_p'*P1*B_p*uu+ x(1:2,i)'*C_p'*L'*P1*L*C_p*x(1:2,i);
        rho1=2*rho1;
        z1(i+1)= z1(i) + dt*( -z1(i) + rho1);
        i=i+1;%disp(i);
        evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);
    end
    while and(and(evo<100,evo>0),i<Nt)
        % we are in D0 or in D1, we jump
        if q(i)==0 
            disp(['jump to the global cont. at t=' num2str(i*dt) ', for i=' num2str(i)]);
        else
            disp(['jump to the local cont. at t=' num2str(i*dt) ', for i=' num2str(i)]);
        end
        x(:,i+1) = x(:,i);
        q(i+1) = 1-q(i);
        z0(i+1) = 0;
        z1(i+1) = z1(i);
        i=i+1;%disp(i);
        evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);
    end
    while and(evo == 0,i<Nt)
        % we are in any set
        disp('we are in ay set: error')
    end
end

% plots
fig=fig+1;figure(fig);hold on;
plot(t,x(1,:),'LineWidth',2);ylabel('x1');xlabel('t');
%title('closed-loop with the hybrid controller');
plot(t,x(2,:),'r:','LineWidth',2);ylabel('x');
hold off

fig=fig+1;figure(fig);hold on
plot(x(1,:),x(2,:),'LineWidth',2);ylabel('x2');xlabel('x1');
hold off

switch inicond
    case 'a1'
print -depsc2 x_1.ps;
    case 'a2'
print -depsc2 x_2.ps;
    case 'b4'
print -depsc2 x_3.ps;        
end

%fig=fig+1;figure(fig);plot(t,x(2,:));ylabel('x2');xlabel('t');
%title('closed-loop with the hybrid controller')
fig=fig+1;figure(fig);plot(t,u,'LineWidth',2);ylabel('u');xlabel('t');
fig=fig+1;figure(fig);hold on
plot(t,u,'LineWidth',2);xlabel('t');
plot(t,z1,'k-.',t,eps1a,'r','LineWidth',2)
axis([0 T -0.03 0.03]);
hold off

switch inicond
    case 'a1'
print -depsc2 uz1_1.ps;
    case 'a2'
        print -depsc2 uz1_2.ps;
end

Npart=[1:20];
%Npart=[1:1500];
fig=fig+1;figure(fig);hold on
plot(t(Npart),u(Npart),'LineWidth',2);xlabel('t');
plot(t(Npart),z0(Npart),'k-.','LineWidth',2);
plot(t(Npart),eps0a*ones(max(Npart),1),'r-','LineWidth',2)
axis([0 t(max(Npart)) -2 1]);
hold off
switch inicond
    case 'a3'
print -depsc2 uz2_2.ps;
end

fig=fig+1;figure(fig);plot(t,q,'*');ylabel('q');xlabel('t');axis([0 T -0.5 1.5]);

switch inicond
    case 'a1'
print -depsc2 q_1.ps;
    case 'a2'
print -depsc2 q_2.ps;
end

fig=fig+1;figure(fig);plot(t,z1,'k',t,eps1a,'r');ylabel('z1');xlabel('t');
fig=fig+1;figure(fig);plot(t,z0,'k',t,eps0a,'r');ylabel('z0');xlabel('t');

if 0
    stoptime=250;
    discrete_time=round(stoptime/dt):Nt;
    fig=fig+1;figure(fig);plot(t(discrete_time),z1(discrete_time),'k',t(discrete_time),eps1a,'r');
    ylabel('z1');xlabel('t');
    fig=fig+1;figure(fig);plot(t(discrete_time),z0(discrete_time),'k',t(discrete_time),eps0a,'r');
    ylabel('z0');xlabel('t');
end

fig0=fig;
disp('done');
return

%%

% simulations with small mesurement noise
fig=fig0;

% set inicond='a2' before

% radius of measurement errors
switch inicond
    case 'a2'
        e=10*10^(-3);
    otherwise
        disp('make sure that inicond has the good value')    
end

% final time
T=100;

% number of discretization
Nt=10000;

% time step
dt=T/Nt;
t=linspace(0,T,Nt);

x=zeros(4,Nt);y =zeros(1,Nt);u=zeros(1,Nt);q=2*ones(1,Nt);z1=zeros(1,Nt);z0=zeros(1,Nt);

i=1;x(:,i)= x0;z1(i)=z10;z0(i)=z00;q(i)=q0;

evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);

if q0 == 0
    disp('we start with the local controller')
else
    disp('we start with the global controller')
end
%keyboard
while i<Nt
    while and(evo >= 1000,i<Nt)
        % we use the local controller
        % keyboard
        %disp('local');
        error=-e + 2*e*rand(2,1);
        xe(1:2,i)=x(1:2,i)+error; % perturbed measurement
        uu= sat(C_l *x(3:4,i) +D_l*C_p* xe(1:2,i),u1);
        u(:,i)=uu;
    x(1:2,i+1) = x(1:2,i) + dt* (A_p * x(1:2,i)+ B_p* uu);
    x(3:4,i+1) = x(3:4,i) + dt* (A_l * x(3:4,i)+  B_l*C_p*x(1:2,i));
    q(i+1) = 0;
    rho0=[uu-C_l*x(3:4,i)-D_l*C_p*x(1:2,i),C_p*x(1:2,i),x(3:4,i)']...
        *N*[ uu-C_l*x(3:4,i)-D_l*C_p*x(1:2,i),C_p*x(1:2,i),x(3:4,i)']';
    rho1= uu*B_p'*P1*P1*B_p*uu+ x(1:2,i)'*C_p'*L'*P1*P1*L*C_p*x(1:2,i);
    rho1=4/eps3*rho1;
    z0(i+1) = z0(i) + dt*( -z0(i)+ rho0);
    z1(i+1)= z1(i) + dt*( -z1(i) + rho1);
    i=i+1;%disp(i);
    evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);
    end
    while and(and(evo<1000,evo>=100),i<Nt)
        % we use the global controller
        % disp('global');
        error=-e + 2*e*rand(2,1);
        xe(1:2,i)=x(1:2,i)+error; % perturbed measurement
        uu=sat(K*C_p*xe(1:2,i),u1);
        u(:,i)=uu;
        x(1:2,i+1) = x(1:2,i) + dt* (A_p * x(1:2,i)+ B_p*uu);
        x(3:4,i+1) = zeros(2,1);
        q(i+1) = 1;
        z0(i+1) = 0;
        rho1= uu*B_p'*P1*P1*B_p*uu+ x(1:2,i)'*C_p'*L'*P1*P1*L*C_p*x(1:2,i);
        rho1=4/eps3*rho1;
        z1(i+1)= z1(i) + dt*( -z1(i) + rho1);
        i=i+1;%disp(i);
        evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);
    end
    while and(and(evo<100,evo>0),i<Nt)
        % we are in D0 or in D1, we jump
        % disp('jump');
        if q(i)==0 
            disp(['jump to the global cont. at t=' num2str(i*dt) ', for i=' num2str(i)]);
        else
            disp(['jump to the local cont. at t=' num2str(i*dt) ', for i=' num2str(i)]);
        end
        x(:,i+1) = x(:,i);
        q(i+1) = 1-q(i);
        z0(i+1) = 0;
        z1(i+1) = z1(i);
        i=i+1;%disp(i);
        evo=evol(z0(i),z1(i),q(i),eps0a,eps1a);
    end
    while and(evo == 0,i<Nt)
        % we are in any set
        disp('error')
    end
end
% 
% % plots
% fig=fig+1;figure(fig);hold on;
% plot(t,x(1,:),'LineWidth',2);ylabel('x1');xlabel('t');
% %title('closed-loop with the hybrid controller');
% plot(t,x(2,:),'r:','LineWidth',2);ylabel('x');
% hold off

fig=fig+1;figure(fig);
plot(x(1,:),x(2,:),'LineWidth',2);ylabel('x2');xlabel('x1');

switch inicond
    case 'a2'
print -depsc2 x_e.ps;
end

%fig=fig+1;figure(fig);plot(t,x(2,:));ylabel('x2');xlabel('t');
%title('closed-loop with the hybrid controller')

fig=fig+1;figure(fig);plot(t,q,'*');ylabel('q');xlabel('t');axis([0 T -0.5 1.5]);
switch inicond
    case 'a2'
print -depsc2 q_e.ps;
end

% %fig=fig+1;figure(fig);plot(t,x(2,:));ylabel('x2');xlabel('t');
% %title('closed-loop with the hybrid controller')
% fig=fig+1;figure(fig);plot(t,u,'LineWidth',2);ylabel('u');xlabel('t');
% fig=fig+1;figure(fig);hold on
% plot(t,u,'LineWidth',2);xlabel('t');
% plot(t,z0,'k-.',t,eps0a,'r','LineWidth',2)
% %axis([0 T -0.03 0.03]);
% hold off
% 
% switch inicond
%     case 'a2'
% print -depsc2 uz1_e.ps;
%    end


% 
% fig=fig+1;figure(fig);plot(t,z1);ylabel('z1');xlabel('t');
% fig=fig+1;figure(fig);plot(t,z0);ylabel('z0');xlabel('t');

disp('done');
